//     PCR1000
//
// Simple Icom PCR-1000 control program
//
// Copyright (c) 1998 Rich Gopstein
// This code may be used for any purpose, if credit is included
// in any product or program derived from this code.
//
// gopstein@bms.com
//


#pragma pack(2)
#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include <System/SerialMgr.h>

#include "pcr.h"
#define PCRAppID    'Pcr1'  
#define PCRDBType   'Data'

static int StartApplication(void);
static Boolean OpenDatabase(void);
static void EventLoop(void);
static void EventLoop(void);
static void StopApplication(void);
static Boolean PCR(EventPtr event);
static void Convert(void);


FieldPtr        fieldptr_text;
DmOpenRef	PCRDB;
char            PCRDBName[] = "PCRDB";
UInt            SerialRef;
int             vol;
int             squelch;
char            volstr[]="J4000\r\n";
char            sqlstr[]="J4100\r\n";
char            freqstr[20];
int             mode;
int             filter;
int             on = 0;

DWord  PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
  int error;

  if (cmd == sysAppLaunchCmdNormalLaunch)
 {

    error = StartApplication();	// Application start code
    if (error) return error;

    EventLoop();	// Event loop

    StopApplication ();	// Application stop code
  }
  return 0;
}

void SetSerialSettings(void) {
  SerSettingsType SerialSettings;

  SerialSettings.baudRate = 9600;
  SerialSettings.ctsTimeout = serDefaultCTSTimeout;
  SerialSettings.flags = serSettingsFlagStopBits1 | 
    serSettingsFlagBitsPerChar8;

  SerSetSettings(SerialRef, &SerialSettings);
}


static int StartApplication(void)
{
  int           error;

  if (SysLibFind("Serial Library", &SerialRef)) {
    ErrDisplay("Error finding Serial Library");
  }

  error = SerOpen(SerialRef, 0, 9600);
  if (error) return error;
  
  SetSerialSettings();

  error = OpenDatabase();
  if (error) return error;

  vol = 4;
  squelch = 0;

  FrmGotoForm(formID_PCR);
}

static Boolean OpenDatabase(void)
{
  UInt          index = 0;
  VoidHand      RecHandle;
  Ptr           RecPointer;
  char          nullstring = 0;

  PCRDB = DmOpenDatabaseByTypeCreator(PCRDBType, PCRAppID, dmModeReadWrite);
  if (!PCRDB) {
    if (DmCreateDatabase(0, PCRDBName, PCRAppID, PCRDBType, false)) return 1;
    PCRDB = DmOpenDatabaseByTypeCreator(PCRDBType, PCRAppID, dmModeReadWrite);
    RecHandle = DmNewRecord(PCRDB, &index, 1);
    RecPointer = MemHandleLock(RecHandle);
    DmWrite(RecPointer, 0, &nullstring, 1);
    MemPtrUnlock(RecPointer); 
    DmReleaseRecord(PCRDB, index, true);  
  }
  return 0;
}

static void EventLoop(void)
{
  short err;
  int formID;
  FormPtr form;
  EventType event;

  do
  {

    EvtGetEvent(&event, 600);

    if (SysHandleEvent(&event)) continue;
    if (MenuHandleEvent((void *)0, &event, &err)) continue;

    if (event.eType == frmLoadEvent)
    {
      formID = event.data.frmLoad.formID;
      form = FrmInitForm(formID);
      FrmSetActiveForm(form);

      FrmSetControlGroupSelection(form, 1, BTN_NFM);
      FrmSetControlGroupSelection(form, 2, BTN_3k);

      switch (formID) 
      {
      case formID_PCR:
        FrmSetEventHandler(form, (FormEventHandlerPtr) PCR);
        break;
      }
    } 
  FrmDispatchEvent(&event);
  } while(event.eType != appStopEvent);
}

static void StopApplication(void)
{
  FldSetTextHandle(fieldptr_text, NULL);
  DmReleaseRecord(PCRDB, 0, false);
  DmCloseDatabase(PCRDB);
  SerClose(SerialRef);
}

static void sendCommand(CharPtr cmd) {
  Err error;

  SerSend(SerialRef, cmd, StrLen(cmd), &error);
  ErrNonFatalDisplayIf( (error!=0),
      "Timed out when trying to send data)");
  if (error)					
    SerClearErr(SerialRef);	
}

static Boolean PCR(EventPtr event)
{
  FormPtr   form;
  int       handled = 0;
  Err error;

  switch (event->eType) 
    {
    case frmOpenEvent:
      form = FrmGetActiveForm();
      fieldptr_text = FrmGetObjectPtr(form, FrmGetObjectIndex(form, fieldID_text));
      FldSetTextHandle (fieldptr_text, (Handle) DmGetRecord(PCRDB, 0));
      FrmDrawForm(form);
      handled = 1;
      break;
    
    case ctlSelectEvent:  // A control button was pressed and released;
      if (event->data.ctlEnter.controlID == buttonID_setfreq) {
	Convert();
	handled = 1;
      } else if (event->data.ctlEnter.controlID == buttonID_on) {
        sendCommand("H101\r\n");
        on = 1;
	handled = 1;
      } else if (event->data.ctlEnter.controlID == buttonID_off) {
	sendCommand("H100\r\n");
	on = 0;
	handled = 1;
      } else if (event->data.ctlEnter.controlID == buttonID_volup) {
	vol++;
	if (vol > 9) vol=9;
        volstr[3] = '0'+vol;
	sendCommand(volstr);
	handled = 1;
      } else if (event->data.ctlEnter.controlID == buttonID_voldn) {
	vol --;
	if (vol < 0) vol=0;
        volstr[3] = '0'+vol;
	sendCommand(volstr);
	handled = 1;
      } else if (event->data.ctlEnter.controlID == buttonID_sqlup) {
	squelch++;
	if (squelch > 9) vol=9;
        sqlstr[3] = '0'+squelch;
	sendCommand(sqlstr);
	handled = 1;
      } else if (event->data.ctlEnter.controlID == buttonID_sqldn) {
	squelch --;
	if (squelch < 0) squelch=0;
        sqlstr[3] = '0'+squelch;
	sendCommand(sqlstr);
	handled = 1;
      } else {
// must have been a mode/filter change;
	Convert();
	handled = 1;
      }

    break;


  case menuEvent:
    switch (event->data.menu.itemID)
    {
    case menuitemID_convert:
      Convert();
      break;
    case menuitemID_about:
      FrmAlert(alertID_about);
      break;
    case menuitemID_copy:
      FldCopy(fieldptr_text);   // user has to select (highllight) the text to copy
      break;
    case menuitemID_paste:
      FldPaste(fieldptr_text);
      break;
    }
    handled = 1;
    break;

  case nilEvent:
    if (on) {
      sendCommand("H101\r\n");  // keepalive;
    }

    handled = 1;
    break;

  }
  return handled;
}

static int setfreq(CharPtr freq, int lenx) {

  Err       error;
  int       filter;
  int       mode;
  int       i;
  char      chr;
  int       len;

  len = StrLen(freq);

  if (len > 7) return 1;

  StrCopy(freqstr, "K00000000000000000\r\n");

  for (i=0; i<len; i++) {
    chr = freqstr[8-i] = freq[len-(i+1)];
    if ((chr > '9') || (chr < '0')) return 1;    
  }

  mode = FrmGetObjectId(FrmGetActiveForm(), 
      FrmGetControlGroupSelection(FrmGetActiveForm(), 1));

  filter = FrmGetObjectId(FrmGetActiveForm(), 
      FrmGetControlGroupSelection(FrmGetActiveForm(), 2));

  mode -= MODE_BASE;
  filter -= FILTER_BASE;

  freqstr[13] = '0'+mode;
  freqstr[15] = '0'+filter;

  SerSend(SerialRef, freqstr, StrLen(freqstr), &error);
  ErrNonFatalDisplayIf( (error!=0),
    "Timed out when trying to send data)");
  if (error)					
    SerClearErr(SerialRef);	
					
  return 0;
}


static void Convert(void)
{
  int  textlength;
  CharPtr  textpointer;

  textpointer = FldGetTextPtr(fieldptr_text);
  textlength = FldGetTextLength(fieldptr_text);


  if (textlength)
  {
    if (setfreq(textpointer, textlength)) {
      FrmAlert(alertID_errornullstring);
    }
  } else {
    FrmAlert(alertID_errornullstring);
  }
}

